"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveSchemas = resolveSchemas;
const formatName_1 = require("../filters/format/formatName");
const formatOperation_1 = require("../filters/format/formatOperation");
const userType_1 = require("../schemas/userType");
const removeReadWriteOnly_1 = require("./removeReadWriteOnly");
/**
 * This function tries to resolve the Schemas of a given OpenAPI document.
 * @param api The OpenAPI document for which to resolve the Schemas.
 * @param isService
 * If true the UserType schema is added and method path will be used for
 * naming. Otherwise the operationId will be used for naming.
 * @returns
 * The resolved Schemas with additional properties 'x-name', 'x-standalone'
 * and 'x-location'
 */
function resolveSchemas(inputApi, isService = true) {
    const extendedSchemas = [];
    const api = JSON.parse(JSON.stringify(inputApi));
    // Add UserType schema
    if (isService) {
        extendedSchemas.push({
            ...userType_1.userTypeSchema,
            'x-standalone': true,
            'x-name': 'UserType',
            'x-location': `#/components/schemas/user_type`,
            'x-service-name': 'Utility',
            'x-schema-type': 'all',
        });
    }
    // Search in components
    if (api.components)
        extendedSchemas.push(...parseComponents(api.components));
    // Search in paths
    if (api.paths)
        extendedSchemas.push(...parsePaths(api.paths, isService));
    const _extendedSchemas = JSON.parse(JSON.stringify(extendedSchemas));
    for (const extendedSchema of _extendedSchemas) {
        const requestSchema = (0, removeReadWriteOnly_1.removeReadOnly)(extendedSchema);
        const responseSchema = (0, removeReadWriteOnly_1.removeWriteOnly)(extendedSchema);
        requestSchema['x-schema-type'] = 'request';
        responseSchema['x-schema-type'] = 'response';
        requestSchema['x-name'] += 'Request';
        responseSchema['x-name'] += 'Response';
        requestSchema['x-location'] += '_request';
        responseSchema['x-location'] += '_response';
        extendedSchemas.push(...[requestSchema, responseSchema]);
    }
    return extendedSchemas;
}
function parseComponents(components) {
    const extendedSchemas = [];
    // Search for schemas in components
    if (components.schemas)
        extendedSchemas.push(...parseComponentsSchemas(components.schemas));
    // Search for parameters in components
    if (components.parameters)
        extendedSchemas.push(...parseComponentsParameters(components.parameters));
    return extendedSchemas;
}
function parseComponentsSchemas(schemas) {
    const extendedSchemas = [];
    for (const schemaName in schemas) {
        const name = (0, formatName_1.formatName)(schemas[schemaName].title ?? 'MISSING');
        extendedSchemas.push({
            ...schemas[schemaName],
            'x-standalone': true,
            'x-name': name,
            'x-location': `#/components/schemas/${schemaName}`,
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            'x-service-name': schemas[schemaName]['x-service-name'],
            'x-schema-type': 'all',
        });
    }
    return extendedSchemas;
}
function parseComponentsParameters(parameters) {
    const extendedSchemas = [];
    for (const parameterName in parameters) {
        const parameter = parameters[parameterName];
        const schema = parameter.schema;
        extendedSchemas.push({
            ...schema,
            'x-standalone': false,
            'x-name': (0, formatName_1.formatName)(parameter.name),
            'x-location': `#/components/parameters/${parameterName}/schema`,
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            'x-service-name': schema['x-service-name'],
            'x-schema-type': 'all',
        });
    }
    return extendedSchemas;
}
function parsePaths(paths, isService) {
    const extendedSchemas = [];
    for (const path in paths) {
        const pathItem = paths[path];
        extendedSchemas.push(...parsePathItem(pathItem, path, isService));
    }
    return extendedSchemas;
}
function parsePathItem(pathItem, path, isService) {
    const extendedSchemas = [];
    for (const method of Object.keys(pathItem)) {
        const operation = pathItem[method];
        const requestBody = operation.requestBody;
        // Add requestBody schema if present
        if (requestBody) {
            const extendedSchema = parseRequestBody(requestBody, operation, path, method, isService);
            if (extendedSchema)
                extendedSchemas.push(extendedSchema);
        }
        // Search for response schemas
        if (operation.responses)
            extendedSchemas.push(...parseResponses(operation.responses, operation, path, method, isService));
    }
    return extendedSchemas;
}
function parseRequestBody(requestBody, operation, path, method, isService) {
    const schema = requestBody.content['application/json'].schema;
    return schema
        ? {
            ...schema,
            'x-standalone': false,
            'x-name': isService
                ? (0, formatOperation_1.formatOperation)(path, method) + 'RequestBody'
                : (0, formatName_1.formatName)(operation.operationId ?? '', false) + 'Body',
            'x-location': `#/paths/${path}/${method}/requestBody/content/application/json/schema`,
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            'x-service-name': operation['x-service-name'],
            'x-schema-type': 'all',
        }
        : undefined;
}
function parseResponses(responses, operation, path, method, isService) {
    const extendedSchemas = [];
    for (const status in responses) {
        const response = responses[status];
        const content = response.content;
        if (content && content['application/json'].schema) {
            extendedSchemas.push({
                ...content['application/json'].schema,
                'x-standalone': false,
                'x-name': isService
                    ? (0, formatOperation_1.formatOperation)(path, method) + 'Response' + status
                    : (0, formatName_1.formatName)(operation.operationId ?? '', false) + 'Response' + status,
                'x-location': `#/paths/${path}/${method}/responses/${status}/content/application/json/schema`,
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                'x-service-name': operation['x-service-name'],
                'x-schema-type': 'all',
            });
        }
        if (response.headers)
            extendedSchemas.push(...parseHeaders(response.headers, operation, path, method, status, isService));
    }
    return extendedSchemas;
}
function parseHeaders(headers, operation, path, method, status, isService) {
    const extendedSchemas = [];
    for (const headerName in headers) {
        const header = headers[headerName];
        if (header.schema) {
            extendedSchemas.push({
                ...header.schema,
                'x-standalone': false,
                'x-name': isService
                    ? (0, formatOperation_1.formatOperation)(path, method) + 'Header' + (0, formatName_1.formatName)(headerName)
                    : (0, formatName_1.formatName)(operation.operationId ?? '', false) + (0, formatName_1.formatName)(headerName),
                'x-location': `#/paths/${path}/${method}/responses/${status}/headers/${headerName}/schema`,
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                'x-service-name': operation['x-service-name'],
                'x-schema-type': 'all',
            });
        }
    }
    return extendedSchemas;
}
//# sourceMappingURL=resolveSchemas.js.map